# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import jahwidgets
import jahstate
import utilities
import openpluginlib
import os.path

#
# Channel property observer
#

class channel_observer( openpluginlib.observer ):
	def __init__( self, prop, update ):
		openpluginlib.observer.__init__( self )
		self.prop = prop
		self.update = update
		
	def updated( self, subject ):
		self.update( self.prop )

#
# The header widget
#

class widget:
	def __init__( self, parent, state ):
		self.state = state
		self.handle = jahwidgets.create( utilities.get_absolute_path( "header.ui" ), parent )
		self.state.register_media_preparation( self.prepare_media )

		# Set up the active media listener
		self.state.register_media_listener( self )

		# Ignore flag to allow asset and combo box syncing without refreshing
		self.ignore_cb_change = False

		# Set up the preview mode combo box
		# Note: In order to preserve display order, the keys are not extracted from the dictionary
		self.preview_keys = [ "A", "B", "AB", "A|B" ]
		self.preview_modes = { }
		self.preview_modes[ self.preview_keys[ 0 ] ] = "preview_a"
		self.preview_modes[ self.preview_keys[ 1 ] ] = "preview_b"
		self.preview_modes[ self.preview_keys[ 2 ] ] = "preview_ab"
		self.preview_modes[ self.preview_keys[ 3 ] ] = "preview_ab_plus"
		self.presentation_preview = utilities.create_property_and_attach( self.handle,
										  "cb_preview:currentItem",
										  utilities.call_method_observer( self.preview_selected ) )
		jahwidgets.set_value( "cb_preview:items", self.handle, openpluginlib.any( self.preview_keys ) )

		# Properties and handlers for changes to RGBA
		# Note: In order to preserve display order, the keys are not extracted from the dictionary
		self.colour_keys = [ "All", "Red", "Green", "Blue", "Alpha" ]
		self.colour_modes = { }
		self.colour_modes[ self.colour_keys[ 0 ] ] = [ 1.0, 1.0, 1.0, 1.0 ]
		self.colour_modes[ self.colour_keys[ 1 ] ] = [ 1.0, 0.0, 0.0, 1.0 ]
		self.colour_modes[ self.colour_keys[ 2 ] ] = [ 0.0, 1.0, 0.0, 1.0 ]
		self.colour_modes[ self.colour_keys[ 3 ] ] = [ 0.0, 0.0, 1.0, 1.0 ]
		self.colour_modes[ self.colour_keys[ 4 ] ] = [ 0.0, 0.0, 0.0, 1.0 ]
		self.presentation_colour = utilities.create_property_and_attach( self.handle,
										 "cb_colour:currentItem",
										 utilities.call_method_observer( self.colour_selected ) )
		jahwidgets.set_value( "cb_colour:items", self.handle, openpluginlib.any( self.colour_keys ) )

		# Set up the presentation combo box
		# Note: In order to preserve display order, the keys are not extracted from the dictionary
		self.fill_keys = [ "Fill Both", "Vertical", "Horizontal", "Native" ]
		self.fill_modes = { }
		self.fill_modes[ self.fill_keys[ 0 ] ] = "scale_fill"
		self.fill_modes[ self.fill_keys[ 1 ] ] = "scale_vert"
		self.fill_modes[ self.fill_keys[ 2 ] ] = "scale_horiz"
		self.fill_modes[ self.fill_keys[ 3 ] ] = "scale_off"
		self.presentation_fill = utilities.create_property_and_attach( self.handle,
									       "cb_fill:currentItem",
									       utilities.call_method_observer( self.fill_selected ) )
		jahwidgets.set_value( "cb_fill:items", self.handle, openpluginlib.any( self.fill_keys ) )

		# Set up the sample aspect ratio combo box
		# Note: In order to preserve display order, the keys are not extracted from the dictionary
		self.sar_keys = [ "Default", "1:1", "NTSC 4:3", "NTSC 16:9", "PAL 4:3", "PAL 16:9" ]
		self.sar_modes = { }
		self.sar_modes[ self.sar_keys[ 0 ] ] = [ 0, 0 ]
		self.sar_modes[ self.sar_keys[ 1 ] ] = [ 1, 1 ]
		self.sar_modes[ self.sar_keys[ 2 ] ] = [ 10, 11 ]
		self.sar_modes[ self.sar_keys[ 3 ] ] = [ 40, 33 ]
		self.sar_modes[ self.sar_keys[ 4 ] ] = [ 59, 54 ]
		self.sar_modes[ self.sar_keys[ 5 ] ] = [ 118, 81 ]
		self.presentation_sar = utilities.create_property_and_attach( self.handle,
									      "cb_sar:currentItem",
									      utilities.call_method_observer( self.sar_selected ) )
		jahwidgets.set_value( "cb_sar:items", self.handle, openpluginlib.any( self.sar_keys ) )

		# Set up the rescaling combo box
		# Note: In order to preserve display order, the keys are not extracted from the dictionary
		self.rescale_keys = [ "Default", "320x240", "640x480", "720x576", "1024x768", "1280x1024", "1440x1080", "1600x1200" ]
		self.rescale_modes = { }
		self.rescale_modes[ self.rescale_keys[ 0 ] ] = 0
		self.rescale_modes[ self.rescale_keys[ 1 ] ] = 240
		self.rescale_modes[ self.rescale_keys[ 2 ] ] = 480
		self.rescale_modes[ self.rescale_keys[ 3 ] ] = 576
		self.rescale_modes[ self.rescale_keys[ 4 ] ] = 768
		self.rescale_modes[ self.rescale_keys[ 5 ] ] = 1024
		self.rescale_modes[ self.rescale_keys[ 6 ] ] = 1080
		self.rescale_modes[ self.rescale_keys[ 7 ] ] = 1200
		self.presentation_rescale = utilities.create_property_and_attach( self.handle,
									      "cb_rescale:currentItem",
									      utilities.call_method_observer( self.rescale_selected ) )
		jahwidgets.set_value( "cb_rescale:items", self.handle, openpluginlib.any( self.rescale_keys ) )

		# Set up the frame rate combo box
		# Note: In order to preserve display order, the keys are not extracted from the dictionary
		self.fps_keys = [ "Default", "1/5 fps", "1 fps", "12 fps", "15 fps", "24 fps", "PAL", "NTSC", "30 fps", "50 fps", "60 fps", "120 fps", "180 fps" ]
		self.fps_modes = { }
		self.fps_modes[ self.fps_keys[ 0 ] ] = [ 0, 0 ]
		self.fps_modes[ self.fps_keys[ 1 ] ] = [ 1, 5 ]
		self.fps_modes[ self.fps_keys[ 2 ] ] = [ 1, 1 ]
		self.fps_modes[ self.fps_keys[ 3 ] ] = [ 12, 1 ]
		self.fps_modes[ self.fps_keys[ 4 ] ] = [ 15, 1 ]
		self.fps_modes[ self.fps_keys[ 5 ] ] = [ 24, 1 ]
		self.fps_modes[ self.fps_keys[ 6 ] ] = [ 25, 1 ]
		self.fps_modes[ self.fps_keys[ 7 ] ] = [ 30000, 1001 ]
		self.fps_modes[ self.fps_keys[ 8 ] ] = [ 30, 1 ]
		self.fps_modes[ self.fps_keys[ 9 ] ] = [ 50, 1 ]
		self.fps_modes[ self.fps_keys[ 10 ] ] = [ 60, 1 ]
		self.fps_modes[ self.fps_keys[ 11 ] ] = [ 120, 1 ]
		self.fps_modes[ self.fps_keys[ 12 ] ] = [ 180, 1 ]
		self.presentation_fps = utilities.create_property_and_attach( self.handle,
									      "cb_fps:currentItem",
									      utilities.call_method_observer( self.fps_selected ) )
		jahwidgets.set_value( "cb_fps:items", self.handle, openpluginlib.any( self.fps_keys ) )

		# Fullscreen
		self.fullscreen_prop = openpluginlib.property( openpluginlib.createKeyFromString( "fullscreen" ) )
		jahwidgets.connect( "fullscreen:on", self.handle, self.fullscreen_prop )
		self.fullscreen_prop.attach( channel_observer( self.fullscreen_prop, self.fullscreen_update ) )

		# Deinterlace
		self.deinterlace_keys = [ "Off", "On" ]
		self.deinterlace_modes = { }
		self.deinterlace_modes[ self.deinterlace_keys[ 0 ] ] = False
		self.deinterlace_modes[ self.deinterlace_keys[ 1 ] ] = True

		self.presentation_deinterlace = utilities.create_property_and_attach( self.handle, "pb_deinterlace:on",
									    utilities.call_method_observer( self.deinterlace_selected ) )

		# Lock
		self.lock_prop = openpluginlib.property( openpluginlib.createKeyFromString( "pb_lock" ) )
		jahwidgets.connect( "pb_lock:on", self.handle, self.lock_prop )

		# Set up the defaults
		self.state.set_preview( "preview_a" )
		self.state.set_presentation( "scale_fill" )

		# Add a dummy set on a label
		self.file_title_prop = openpluginlib.property( openpluginlib.createKeyFromString( "file_title" ) )
		jahwidgets.connect( "file_title:text", self.handle, self.file_title_prop )
		self.file_title_prop.set( "" )

	def fullscreen_update( self, prop ):
		self.state.player.toggle_full_screen( )

	def stored_value( self, asset, keys, name ):
		try:
			if asset is None: 
				return 0
			elif str( asset.find( name ).first ) != "":
				return keys.index( str( asset.find( name ).second ) )
			else:
				return 0
		except:
			return 0

	def notify_media_change( self, media ):
		# Show the name of the asset
		if media.asset is not None and not str( media.asset.find( "name" ).second ) == "":
			self.file_title_prop.set( str( media.asset.find( "name" ).second ) )
		else:
			self.file_title_prop.set( "" )

		# Need to ensure that the sar cb reads correctly, but we don't want to force a refresh
		if not self.lock_prop.valueAsBool( ):
			self.ignore_cb_change = True
			self.presentation_sar.set( self.stored_value( media.asset, self.sar_keys, "jahplayer::sar" ) )
			self.presentation_rescale.set( self.stored_value( media.asset, self.rescale_keys, "jahplayer::rescale" ) )
			self.presentation_fps.set( self.stored_value( media.asset, self.fps_keys, "jahplayer::fps" ) )
			# TODO: Find out why this is failing?
			#value = 1 == self.stored_value( media.asset, self.deinterlace_keys, "jahplayer::deinterlace" )
			#self.presentation_deinterlace.set( value )
			# work around - not reflected in ui
			media.deinterlace = self.stored_value( media.asset, self.deinterlace_keys, "jahplayer::deinterlace" )
			self.ignore_cb_change = False

		if self.state.player.speed == 0:
			self.state.player.refresh( )

	def prepare_media( self, media ):
		if media.asset is not None:
			if self.lock_prop.valueAsBool( ):
				self.assign_sar( media, self.presentation_sar.valueAsInt( ) )
				self.assign_rescale( media, self.presentation_rescale.valueAsInt( ) )
				self.assign_fps( media, self.presentation_fps.valueAsInt( ) )
				self.assign_deinterlace( media, self.presentation_deinterlace.valueAsBool( ) )
			else:
				self.assign_sar( media, self.stored_value( media.asset, self.sar_keys, "jahplayer::sar" ) )
				self.assign_rescale( media, self.stored_value( media.asset, self.rescale_keys, "jahplayer::rescale" ) )
				self.assign_fps( media, self.stored_value( media.asset, self.fps_keys, "jahplayer::fps" ) )
				# Temporary - don't serialise deinterlace settings (issues with setting state on the toggle buttons
				# self.assign_deinterlace( media, 1 == self.stored_value( media.asset, self.deinterlace_keys, "jahplayer::deinterlace" ) )
				self.assign_deinterlace( media, self.presentation_deinterlace.valueAsBool( ) )

			media.in_point = 0
			media.out_point = 1
			if not str( media.asset.find( "frames" ).first ) == "":
				media.out_point = int( str( media.asset.find( "frames" ).second ) ) 
			if not str( media.asset.find( "jahplayer::in" ).first ) == "":
				media.in_point = int( str( media.asset.find( "jahplayer::in" ).second ) ) 
			if not str( media.asset.find( "jahplayer::out" ).first ) == "":
				media.out_point = int( str( media.asset.find( "jahplayer::out" ).second ) ) 

	def assign_sar( self, media, index ):
		sar = self.sar_modes[ self.sar_keys[ index ] ]
		media.set_sar( sar[ 0 ], sar[ 1 ] )

	def assign_rescale( self, media, index ):
		rescale = self.rescale_modes[ self.rescale_keys[ index  ] ]
		media.cache_height = rescale

	def assign_fps( self, media, index ):
		fps = self.fps_modes[ self.fps_keys[ index ] ]
		media.set_fps( fps[ 0 ], fps[ 1 ] )

	def assign_deinterlace( self, media, index ):
		media.deinterlace = index

	def preview_selected( self, subject ):
		self.state.set_preview( self.preview_modes[ self.preview_keys[ self.presentation_preview.valueAsInt( ) ] ] )

	def fill_selected( self, subject ):
		self.state.set_presentation( self.fill_modes[ self.fill_keys[ self.presentation_fill.valueAsInt( ) ] ] )

	def colour_selected( self, subject ):
		colour = self.colour_modes[ self.colour_keys[ self.presentation_colour.valueAsInt( ) ] ]
		self.state.player.set_channels( colour[ 0 ], colour[ 1 ], colour[ 2 ], colour[ 3 ] )
		if self.presentation_colour.valueAsInt( ) == 0:
			self.state.player.set_histogram_channels( 0.299, 0.587, 0.114, 0.0 )
		else:
			if self.presentation_colour.valueAsInt( ) != 4:
				self.state.player.set_histogram_channels( colour[ 0 ], colour[ 1 ], colour[ 2 ], 0.0 )
			else:
				self.state.player.set_histogram_channels( colour[ 0 ], colour[ 1 ], colour[ 2 ], colour[ 3 ] )
			
		self.state.player.refresh( )

	def sar_selected( self, subject ):
		if not self.ignore_cb_change:
			self.assign_sar( self.state.player.active, self.presentation_sar.valueAsInt( ) )
			if self.state.player.active.asset is not None:
				self.state.player.active.asset.set( "jahplayer::sar", self.sar_keys[ self.presentation_sar.valueAsInt( ) ] )
			self.state.player.refresh( )

	def rescale_selected( self, subject ):
		if not self.ignore_cb_change:
			self.assign_rescale( self.state.player.active, self.presentation_rescale.valueAsInt( ) )
			if self.state.player.active.asset is not None:
				self.state.player.active.asset.set( "jahplayer::rescale", self.rescale_keys[ self.presentation_rescale.valueAsInt( ) ] )
			self.state.player.refresh( )

	def fps_selected( self, subject ):
		if not self.ignore_cb_change:
			self.assign_fps( self.state.player.active, self.presentation_fps.valueAsInt( ) )
			if self.state.player.active.asset is not None:
				self.state.player.active.asset.set( "jahplayer::fps", self.fps_keys[ self.presentation_fps.valueAsInt( ) ] )
				self.state.player.speed = self.state.player.speed
			self.state.player.refresh( )

	def deinterlace_selected( self, subject ):
		if not self.ignore_cb_change:
			self.assign_deinterlace( self.state.player.active, self.presentation_deinterlace.valueAsBool( ) )
			if self.state.player.active.asset is not None:
				self.state.player.active.asset.set( "jahplayer::deinterlace", self.deinterlace_keys[ self.presentation_deinterlace.valueAsBool( ) ] )
			self.state.player.refresh( )

